/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Raúl Yeguas <neokore@gmail.com>
 *              Martin Borho <martin@borho.net>
 */
import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1

Component {
    ComposerSheet {
        id: sheet
        objectName: "SettingsSheet"
        title: i18n.tr("Settings")
        contentsHeight: parent.height
        contentsWidth: parent.width

        function fillTempSelector() {
            with(temperatureUnitsSelectorModel) {
                append({ name: "celsiusOption", label: i18n.tr("Celsius")});
                append({ name: "fahrenheitOption", label: i18n.tr("Fahrenheit")});
            }
            temperatureUnitsSelector.selectedIndex = (settings["units"] === "imperial") ? 1 : 0;
        }

        function fillWindSelector() {
            with(windUnitsSelectorModel) {
                append({ name: "kmhOption", label: i18n.tr("Kilometers per hour")});
                append({ name: "mphOption", label: i18n.tr("Miles per hour")});
            }
            windUnitsSelector.selectedIndex = (settings["wind_units"] === "mph") ? 1 : 0;
        }

        function fillPrecipSelector() {
            with(precipitationUnitsSelectorModel) {
                append({ name: "millimetersOption", label: i18n.tr("Millimeters")});
                append({ name: "inchesOption", label: i18n.tr("Inches")});
            }
            precipitationUnitsSelector.selectedIndex = (settings["precip_units"] === "in") ? 1 : 0;
        }

        Component.onCompleted: {
            fillTempSelector();
            fillWindSelector();
            fillPrecipSelector();
            mainView.shortCutsLimited = true;
            Theme.palette.selected.backgroundText = "#656565"
        }

        container: Column {
            anchors.left: parent.left
            anchors.right: parent.right

            Component {
                id: unitsSelectorDelegate
                OptionSelectorDelegate { 
                    text: i18n.tr(label)
                    objectName: name 
                }
            }

            ListModel {
                id: temperatureUnitsSelectorModel
            }

            OptionSelector {
                id: temperatureUnitsSelector
                objectName: "TemperatureUnitsSelector"
                text: i18n.tr("Temperature units")
                delegate: unitsSelectorDelegate
                model: temperatureUnitsSelectorModel
            }

            ListModel {
                id: windUnitsSelectorModel
            }

            OptionSelector {
                id: windUnitsSelector
                objectName: "WindUnitsSelector"
                text: i18n.tr("Wind speed units")
                delegate: unitsSelectorDelegate
                model: windUnitsSelectorModel
            }

            ListModel {
                id: precipitationUnitsSelectorModel
            }

            OptionSelector {
                id: precipitationUnitsSelector
                objectName: "PrecipitationUnitsSelector"
                text: i18n.tr("Precipitation units")
                delegate: unitsSelectorDelegate
                model: precipitationUnitsSelectorModel
                // Precipitation units not available at TWC
                visible: (serviceSelector.selectedIndex !== 0) ?  1 : 0
                opacity: (serviceSelector.selectedIndex !== 0) ?  1 : 0
                Behavior on visible {
                    enabled: (serviceSelector.selectedIndex === 0)
                    NumberAnimation { duration:  1500}
                }
                Behavior on opacity {
                    NumberAnimation {
                        easing: UbuntuAnimation.StandardEasingReverse;
                        duration: UbuntuAnimation.SlowDuration
                    }
                }
            }

            ListModel {
                id: serviceSelectorModel
                ListElement { name: "twcOption"; label: "The Weather Channel" }
                ListElement { name: "owmOption"; label: "Openweathermap" }
            }

            OptionSelector {
                id: serviceSelector
                objectName: "ServiceSelector"
                text: i18n.tr("Weather Service")
                delegate: unitsSelectorDelegate
                model: serviceSelectorModel
                selectedIndex: (settings["service"] === "openweathermap") ? 1 : 0;
            }
        }

        signal cancelSheet()
        onCancelSheet: {
            Theme.palette.selected.backgroundText = "#f4f4e8";
            mainView.shortCutsLimited = false;
            PopupUtils.close(sheet)
        }
        Connections { target: mainView; onEscapeKey: cancelSheet() }

        onConfirmClicked: {
            var refresh_from_storage = false,
                refresh_from_service = false,
                selectedUnit = (temperatureUnitsSelector.selectedIndex === 0) ? "metric" : "imperial",
                selectedWindUnit = (windUnitsSelector.selectedIndex === 0) ? "kmh" : "mph",
                selectedPrecipUnit = (precipitationUnitsSelector.selectedIndex === 0) ? "mm" : "in",
                selectedService = (serviceSelector.selectedIndex === 0) ? "weatherchannel" : "openweathermap";
            // check if temperaure scale was changed
            if(settings["units"] !== selectedUnit) {
                storage.saveSetting("units", selectedUnit);
                refresh_from_storage = true;
            }
            //
            if(settings["wind_units"] !== selectedWindUnit) {
                storage.saveSetting("wind_units", selectedWindUnit);
                refresh_from_storage = true;
            }
            //
            if(settings["precip_units"] !== selectedPrecipUnit) {
                storage.saveSetting("precip_units", selectedPrecipUnit);
                refresh_from_storage = true;
            }
            //
            if(settings["service"] !== selectedService) {
                storage.saveSetting("service", selectedService);
                refresh_from_service = true;
            }
            // handling of other settings here
            // ....

            // a setting was changed, reload settings and refresh the location tabs
            if(refresh_from_storage === true || refresh_from_service === true) {
                storage.getSettings(function(storedSettings) {
                    for(var settingName in storedSettings) {
                        settings[settingName] = storedSettings[settingName];
                    }
                    refreshData(refresh_from_storage, refresh_from_service);
                });
            }
            Theme.palette.selected.backgroundText = "#f4f4e8"
            PopupUtils.close(sheet)
        }

        onCancelClicked: {
            cancelSheet()
        }
    }
}
