# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# Authored by: Martin Borho <martin@borho.net>

"""weather-app autopilot tests."""

import os.path
import sqlite3
import time
import logging
import os
import shutil

from autopilot.input import Mouse, Touch, Pointer
from autopilot.platform import model
from testtools.matchers import Equals, Is, Not
from autopilot.matchers import Eventually

from ubuntuuitoolkit import (
    base,
    emulators as toolkit_emulators
)
from ubuntu_weather_app import emulators
import ubuntu_weather_app

logger = logging.getLogger(__name__)


class WeatherTestCase(base.UbuntuUIToolkitAppTestCase):

    """A common test case class that provides several useful methods for
       weather app tests."""
    platform_model = model()
    if platform_model == 'Desktop':
        scenarios = [('with mouse', dict(input_device_class=Mouse))]
    else:
        scenarios = [('with touch', dict(input_device_class=Touch))]

    local_location = "../../ubuntu-weather-app.qml"
    installed_location = "/usr/share/ubuntu-weather-app/ubuntu-weather-app.qml"

    def setUp(self):
        super(WeatherTestCase, self).setUp()
        self.pointing_device = Pointer(self.input_device_class.create())
        self.launch_app()

    def launch_app(self):
        if os.path.exists(self.local_location):
            logger.debug("Running via local installation")
            self.launch_test_local()
        elif os.path.exists(self.installed_location):
            logger.debug("Running via installed debian package")
            self.launch_test_installed()
        else:
            logger.debug("Running via click package")
            self.launch_test_click()

    def launch_test_local(self):
        self.app = self.launch_test_application(
            base.get_qmlscene_launch_command(),
            self.local_location,
            app_type='qt',
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_test_installed(self):
        self.app = self.launch_test_application(
            base.get_qmlscene_launch_command(),
            self.installed_location,
            app_type='qt',
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_test_click(self):
        self.app = self.launch_click_package(
            "com.ubuntu.weather",
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    """Test if tabs with data are present in tablet mode"""
    def _ensure_tablet_tabs_loaded(self):
        self.assertThat(lambda: self.main_view.wait_select_single(
            'QQuickRectangle', visible=True, objectName='SideLabel'),
            Eventually(Not(Is(None)), timeout=60))

    @property
    def main_view(self):
        return self.app.wait_select_single(emulators.MainView)


class SheetMixin(object):
    """A mixin to to give access to common sheet elements"""

    def _click_sheet_confirm(self):
        """Clicks the confirm button"""
        button = self.main_view.select_single(
            'Button', objectName='confirmButton')
        self.pointing_device.click_object(button)

    def _click_sheet_cancel(self):
        """Clicks the cancel button"""
        button = self.main_view.select_single(
            'Button', objectName='cancelButton')
        self.pointing_device.click_object(button)


class LocationManagerMixin(object):
    """Mixin for LocationManager related methods"""
    def _open_add_location_page(self):
        """Opens the AddLocationPage"""
        # click on listitem to open AddLocationPage
        addCityItem = self.main_view.wait_select_single(
            'Standard', objectName='AddCityListItem')
        self.assertThat(lambda: addCityItem.visible, Eventually(Equals(True)))
        self.pointing_device.move_to_object(addCityItem)
        self.pointing_device.click()

        addLocPage = self.main_view.select_single(
            "DefaultSheet", objectName="AddLocationSheet")
        self.assertThat(addLocPage.visible, Eventually(Equals(True)))


class DatabaseMixin(object):

    """
    Helper functions for dealing with sqlite databases
    """

    app_dir = os.path.expanduser(
        "~/.local/share/com.ubuntu.weather")
    db_dir = os.path.join(app_dir, 'Databases')
    db_file = "34e1e542f2f083ff18f537b07a380071.sqlite"
    db_path = os.path.join(db_dir, db_file)

    json_path = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             '..', 'files'))

    def _execute_sql(self, statement):
        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        logger.debug("Executing sql")
        logger.debug(statement)
        cursor.execute(statement)
        conn.commit()
        conn.close()

    def clean_db(self):
        logger.debug("Removing existing database at %s" % self.db_dir)
        try:
            shutil.rmtree(self.db_dir)
        except OSError:
            logger.debug("Could not remove existing database")
            pass

    def create_blank_db(self):
        self.clean_db()

        # create blank db
        database_tmpl_dir = os.path.join(
            os.path.dirname(ubuntu_weather_app.__file__),
            'Databases')
        logger.debug("Creating blank db on filesystem %s" % self.db_dir)
        if not os.path.exists(self.app_dir):
            os.makedirs(self.app_dir)
        shutil.copytree(database_tmpl_dir, self.db_dir)
        self.assertThat(
            lambda: os.path.exists(self.db_path),
            Eventually(Equals(True)))

        # this needs to stay in sync with Storage.qml enties
        logger.debug("Creating locations and settings tables")
        self._execute_sql("CREATE TABLE IF NOT EXISTS Locations(id INTEGER "
                          "PRIMARY KEY AUTOINCREMENT, data TEXT, date TEXT)")

        self._execute_sql("CREATE TABLE IF NOT EXISTS settings(key TEXT "
                          "UNIQUE, value TEXT)")

    def get_locations_data(self):
        result = []
        json_files = [self.json_path + '/1.json', self.json_path + '/2.json']
        for path in json_files:
            with open(path) as fh:
                json_str = fh.read()
                result.append(json_str.strip())
                fh.close()
        return result

    def add_locations_to_database(self, limit=None):
        locations = self.get_locations_data()
        conn = sqlite3.connect(self.db_path)
        cursor = conn.cursor()
        logger.debug("Adding locations to database")
        for loc_data in locations:
            db_exec = "INSERT INTO Locations(date, data) VALUES('{}', '{}')"
            cursor.execute(db_exec.format(
                int(time.time() * 1000), loc_data))
            conn.commit()
        conn.close()

    def add_units_to_database(self):
        logger.debug("Adding units to database")
        self._execute_sql("INSERT INTO settings(key, value) "
                          "VALUES('units', 'metric'), "
                          "('wind_units', 'kmh'), "
                          "('precip_units', 'mm'),"
                          "('service', 'weatherchannel')")
