# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for managing locations in Weather app"""

from __future__ import absolute_import

from testtools.matchers import Equals, Not
from autopilot.matchers import Eventually
from autopilot.introspection.dbus import StateNotFoundError
from unittest import skip

from ubuntu_weather_app.tests import (
    WeatherTestCase,
    DatabaseMixin,
    SheetMixin,
    LocationManagerMixin
)

from time import sleep
import logging

logger = logging.getLogger(__name__)


class TestLocationManager(WeatherTestCase, DatabaseMixin, SheetMixin,
                          LocationManagerMixin):

    def setUp(self):
        # we want to start with no database
        self.clean_db()
        super(TestLocationManager, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

    def _preload_database_and_launch(self):
        # we want to start with existing data
        logger.debug("Closing app")
        self.doCleanups()
        # wipe the existing database and create a new blank one
        self.create_blank_db()
        # add locations to storage
        logger.debug("Adding fake data to new database")
        self.add_locations_to_database()
        logger.debug("Re-Launching app to introspect")
        super(TestLocationManager, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

    def _open_location_manager(self, kb=False):
        """Opens the location manager"""
        if kb:
            self.keyboard.press_and_release('Ctrl+n')
        else:
            self.main_view.open_toolbar().click_button("EditButton")

        # test location manager becomes visible
        managerSheet = self.main_view.select_single(
            'ComposerSheet', objectName='LocationManagerSheet')
        self.assertThat(managerSheet.visible, Eventually(Equals(True)))

    def _swipe_first_location_to_remove(self):
        """Swipe right to delete the first location"""
        first_location = self.app.wait_select_single(
            "Standard", objectName="existingLocationItem0")
        number_of_locations = self._get_number_of_locations()

        x, y, w, h = first_location.globalRect
        tx = x + (w / 8)
        ty = y + (h / 2)
        self.pointing_device.drag(tx, ty, tx + w / 4 * 3, ty)
        self.assertThat(self._get_number_of_locations,
                        Eventually(Equals(number_of_locations - 1)))

    def _get_number_of_locations(self):
        qqlw = self.app.wait_select_single("ComposerSheet").wait_select_single(
            "QQuickListView")
        return qqlw.count

    def _search_for_city(self, city):
        searchField = self.main_view.wait_select_single(
            "TextField", objectName="SearchField")
        # poll clicking for focus for lack of better method, due to screen
        # switching animation is there a property we can use instead?
        timeout = 0
        while searchField.focus is not True and timeout < 10:
            self.pointing_device.click_object(searchField)
            sleep(1)
            timeout += 1
        self.keyboard.type(city)
        self.assertThat(searchField.text, Eventually(Equals(city)))
        self.keyboard.press_and_release('Enter')
        return searchField

    def _add_location(self, location):
        self._open_add_location_page()

        # insert city name to search for
        self._search_for_city(location)

        # wait for results and click on the first item
        self.main_view.wait_for_activity_to_finish()
        resultsList = self.main_view.select_single(
            'QQuickListView', objectName='SearchResultList')
        self.assertThat(resultsList.visible, Eventually(Equals(True)))
        firstResult = self.main_view.wait_select_single(
            'Label', objectName='searchResult0')
        self.pointing_device.move_to_object(firstResult)
        self.pointing_device.click()

        # LocationManagerPage should be visible and location added
        addedItem = self.main_view.wait_select_single('Label', text=location)
        self.assertThat(addedItem.text, Eventually(Equals(location)))
        self._click_sheet_confirm()

        # back to locations, wait till data is loaded
        self.main_view.wait_for_activity_to_finish()

    def _remove_location(self):
        self._open_location_manager()
        self._swipe_first_location_to_remove()
        self._click_sheet_confirm()

    def _verify_location(self, title, index):
        location = self.main_view.wait_select_single(
            'LocationTab', objectName='LocationTab{}'.format(index))
        self.assertThat(location.title, Equals(title))
        self.assertThat(location.index, Equals(index))
        self.assertThat(location.visible, Equals(True))

    def _wait_for_tab(self):
        loadingPage = self.main_view.get_tabs()
        self.assertThat(loadingPage.visible, Eventually(Equals(True)))

    def test_add_location(self):
        """Adds a location"""

        location = "Paris"
        self._add_location(location)

        # only location is there
        self._verify_location(location, 0)

    def test_add_predefined_location(self):
        """Adds a location"""
        self._open_add_location_page()

        resultsList = self.main_view.select_single(
            'QQuickListView', objectName='SearchResultList')
        self.assertThat(resultsList.visible, Eventually(Equals(True)))
        testResult = self.main_view.wait_select_single(
            'Label', objectName='searchResult4')
        self.pointing_device.move_to_object(testResult)
        self.pointing_device.click()

        # LocationManagerPage should be visible and location added
        location = "Cairo"
        addedItem = self.main_view.wait_select_single(
            'Label', objectName="existingLocation0")
        self.assertThat(addedItem.text, Eventually(Equals(location)))
        self._click_sheet_confirm()

        # back to locations, wait till data is loaded
        self.main_view.wait_for_activity_to_finish()

        # only location is there
        self._verify_location(location, 0)

    def test_search_city(self):
        """Search for cities"""
        self._open_add_location_page()

        # insert city name to search for
        searchField = self._search_for_city("London")

        # wait for results and click on the first item
        self.main_view.wait_for_activity_to_finish()
        resultsList = self.main_view.select_single(
            'QQuickListView', objectName='SearchResultList')
        self.assertThat(resultsList.visible, Eventually(Equals(True)))
        firstResult = self.main_view.wait_select_single(
            "Label", objectName="searchResult0")
        self.assertThat(firstResult.text, Eventually(Equals("London")))

        # regain focus, clear search field and do another search
        searchField = self.main_view.wait_select_single(
            "TextField", objectName="SearchField")
        self.pointing_device.click_object(searchField)
        clear_button = searchField.wait_select_single("AbstractButton")
        self.pointing_device.click_object(clear_button)
        self.assertThat(searchField.text, Eventually(Equals("")))

        self.pointing_device.click_object(searchField)
        self.keyboard.type("Hamburg")
        self.keyboard.press_and_release('Enter')
        self.main_view.wait_for_activity_to_finish()
        self.assertThat(resultsList.visible, Eventually(Equals(True)))
        firstResult = self.main_view.wait_select_single(
            "Label", objectName="searchResult0")
        self.assertThat(firstResult.text, Eventually(Equals("Hamburg")))

    def test_cancel_adding_location(self):
        """Cancel the cities search"""
        self._open_add_location_page()
        self._click_sheet_cancel()
        locationList = self.main_view.wait_select_single(
            'QQuickListView', objectName='LocationList')
        self.assertThat(locationList.visible, Eventually(Equals(True)))

    def test_no_location_found(self):
        """Tests if label with 'No city found' is shown """
        self._open_add_location_page()

        # insert city name to search for
        self._search_for_city("UbuntuCity")

        # wait for result and look if label is shown
        self.main_view.wait_for_activity_to_finish()
        errorLabel = self.main_view.select_single('Label',
                                                  objectName='noCityError')
        self.assertThat(errorLabel.visible, Eventually(Equals(True)))

    def test_location_lookup(self):
        """Tests the location lookup"""
        # start lookup
        lookupItem = self.main_view.wait_select_single(
            'Standard', objectName='LocationLookupItem')
        self.pointing_device.click_object(lookupItem)

        # wait for result
        self.main_view.wait_for_activity_to_finish()

        # check a location was found
        location = self.main_view.wait_select_single(
            'Label', objectName='CurrentLocationLabel')
        self.assertThat(location.text, Eventually(Not(Equals(""))))
        location_name = location.text

        # add location
        add_button = self.main_view.wait_select_single(
            'Button', objectName='LookupItemAddButton')
        self.pointing_device.click_object(add_button)

        # LocationManagerPage should be visible and location added
        addedItem = self.main_view.wait_select_single(
            'Label', objectName="existingLocation0")
        self.assertThat(addedItem.text, Eventually(Equals(location_name)))
        self._click_sheet_confirm()

        # back to locations, wait till data is loaded
        self.main_view.wait_for_activity_to_finish()

        # only location is there
        self._verify_location(location_name, 0)

    def test_remove_location(self):
        """Removes location"""
        self._preload_database_and_launch()

        # wait till data is loaded
        self._wait_for_tab()

        # count locations at start
        tabObjects = self.main_view.select_many('LocationTab')
        tabsSumStart = len(tabObjects)

        # go to the location manager and remove 1st location
        self._remove_location()

        # back to locations, only one is left
        self.main_view.wait_for_activity_to_finish()
        tabObjects = lambda: len(self.main_view.select_many('LocationTab'))
        self.assertThat(tabObjects, Eventually(Equals(tabsSumStart - 1)))

    def test_cancel_remove_location(self):
        """Cancels removing of location"""
        self._preload_database_and_launch()

        # wait data is loaded
        self._wait_for_tab()

        # count locations at start
        tabObjects = self.main_view.select_many('LocationTab')
        tabsSumStart = len(tabObjects)

        # go to the location manager, remove location, but click cancel
        self._open_location_manager()
        self._swipe_first_location_to_remove()
        self._click_sheet_cancel()

        # back to locations, no loction is removed
        self._wait_for_tab()
        tabObjects = lambda: len(self.main_view.select_many('LocationTab'))
        self.assertThat(tabObjects, Eventually(Equals(tabsSumStart)))

    def test_del_add_bug(self):
        """Tests if removing and add a location in one action works
           https://bugs.launchpad.net/ubuntu-weather-app/+bug/1230297"""

        self._preload_database_and_launch()

        # remove a location
        self._remove_location()

        # add a location
        self._open_location_manager()
        self._add_location("Cairo")

        # back to locations, wait till data is loaded
        self.main_view.wait_for_activity_to_finish()

        # count the tabs
        tabObjects = self.main_view.select_many('LocationTab')
        self.assertThat(lambda: len(tabObjects), Eventually(Equals(2)))
        self.assertThat(
            self.main_view.select_single('LocationTab', index=1).title,
            Eventually(Equals("Cairo")))

    @skip("skipped because of Bug #1279073")
    def test_open_location_manager_keyboard(self):
        if self.platform_model != "Desktop":
            return
        self._preload_database_and_launch()
        self._open_location_manager(kb=True)

        # close sheet by keyboard and test
        self.keyboard.press_and_release('Escape')
        self.assertRaises(
            StateNotFoundError,
            lambda: self.main_view.select_single('ComposerSheet'))

    @skip("skipped because of Bug #1279073")
    def test_add_location_keyboard(self):
        if self.platform_model != "Desktop":
            return
        self._preload_database_and_launch()
        self._open_location_manager(kb=True)
        self._open_add_location_page()

        # close sheet by keyboard and test all sheets are closed
        self.keyboard.press_and_release('Escape')
        self.assertRaises(
            StateNotFoundError,
            lambda: self.main_view.select_single('DefaultSheet'))
        self.assertRaises(
            StateNotFoundError,
            lambda: self.main_view.select_single('ComposerSheet'))
