# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# Authored by: Martin Borho <martin@borho.net>

"""Tests for Weather app"""

from __future__ import absolute_import

from testtools.matchers import Equals, Is, Not
from autopilot.matchers import Eventually
from unittest import skip

import logging

from ubuntu_weather_app.tests import WeatherTestCase, DatabaseMixin

logger = logging.getLogger(__name__)


class TestMainView(WeatherTestCase, DatabaseMixin):
    """Tests creating weather"""

    """ This is needed to wait for the application to start.
        In the testfarm, the application may take some time to show up."""
    def setUp(self):
        # we want to start with fake location data
        self.create_blank_db()
        logger.debug("Adding fake data to new database")
        self.add_locations_to_database()
        logger.debug("Re-Launching app to introspect")
        super(TestMainView, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

    def _switch_tabs(self, kb=False, back=True):
        """Test start of app with two locations and switch from first to
        second tab"""

        # first tab should be visible after start
        tabs = self.main_view.get_tabs()
        first_tab = self.main_view.select_single(
            'LocationTab', index=0)
        second_tab = self.main_view.select_single(
            'LocationTab', index=1)
        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(0)))
        self.assertThat(first_tab.visible, Eventually(Equals(True)))
        self.assertThat(second_tab.visible, Eventually(Equals(False)))
        self.assertThat(first_tab.title, Eventually(Equals("Hamburg")))

        # switch to next tab
        if kb and not back:
            self.keyboard.press_and_release('Ctrl+Tab')
        elif kb and back:
            self.keyboard.press_and_release('Ctrl+Shift+Tab')
        else:
            self.main_view.switch_to_tab_by_index(1)

        self.assertThat(tabs.selectedTabIndex, Eventually(Equals(1)))
        self.assertThat(first_tab.visible, Eventually(Equals(False)))
        self.assertThat(second_tab.visible, Eventually(Equals(True)))
        self.assertThat(second_tab.title, Eventually(Equals("London")))

    def test_switch_tabs(self):
        """
        Test start of app with two locations and switch from first to second
        tab
        """
        self._switch_tabs()

    @skip("skipped because of Bug #1279073")
    def test_switch_tabs_keyboard(self):
        """
        Test start of app with two locations and switch from first to second
        tab
        """
        if self.platform_model != "Desktop":
            return
        self._switch_tabs(kb=True)

    @skip("skipped because of Bug #1279073")
    def test_switch_tabs_keyboard_back(self):
        """
        Test start of app with two locations and switch from first to second
        tab
        """
        if self.platform_model != "Desktop":
            return
        self._switch_tabs(kb=True, back=True)

    def _refresh(self, kb=False):
        # get the dates from the test data
        if self.main_view.wideAspect:
            self._ensure_tablet_tabs_loaded()
        self.assertThat(lambda: self.main_view.select_many(
            'Label', objectName='DayDateLabel'), Eventually(Not(Is(None))))
        curr_dates = self.main_view.select_many('Label',
                                                objectName='DayDateLabel')
        tab1_curr_date = curr_dates[0].text
        tab2_curr_date = curr_dates[2].text

        # refresh and wait till loading has finished
        if kb:
            self.keyboard.press_and_release('Ctrl+r')
        else:
            self.main_view.open_toolbar().click_button("RefreshButton")
        self.main_view.wait_for_activity_to_finish()

        # get the data from the refreshed tabs, have to be new
        self.assertThat(lambda: self.main_view.select_many(
            'Label', objectName='DayDateLabel'),
            Eventually(Not(Is(None)), timeout=60))
        refreshed_dates = self.main_view.select_many('Label',
                                                     objectName='DayDateLabel')
        self.assertNotEqual(tab1_curr_date, refreshed_dates[0].text)
        self.assertNotEqual(tab2_curr_date, refreshed_dates[2].text)

    def test_refresh_tabs(self):
        self._refresh()

    @skip("skipped because of Bug #1279073")
    def test_refresh_tabs_keyboard(self):
        if self.platform_model != "Desktop":
            return
        self._refresh(kb=True)

    @skip("skipped because of Bug #1279073")
    def test_day_navigation_keyboard(self):
        # get the dates from the test data
        if self.platform_model != "Desktop":
            return
        self.assertThat(lambda: self.main_view.select_many(
            'QQuickListView', objectName='DailyForecastList'),
            Eventually(Not(Is(None))))
        dayLists = self.main_view.select_many('QQuickListView',
                                              objectName='DailyForecastList')
        self.assertThat(dayLists[1].visible, Eventually(Equals(True)))
        self.assertThat(dayLists[1].currentIndex, Eventually(Equals(0)))

        # navigate to to overmorrow
        self.keyboard.press_and_release('Down')
        self.keyboard.press_and_release('Down')
        dayLists = self.main_view.select_many('QQuickListView',
                                              objectName='DailyForecastList')
        self.assertThat(dayLists[1].visible, Eventually(Equals(True)))
        self.assertThat(dayLists[1].currentIndex, Eventually(Equals(2)))

        # navigate to tomorrow
        self.keyboard.press_and_release('Up')
        dayLists = self.main_view.select_many('QQuickListView',
                                              objectName='DailyForecastList')
        self.assertThat(dayLists[1].visible, Eventually(Equals(True)))
        self.assertThat(dayLists[1].currentIndex, Eventually(Equals(1)))

    @skip("skipped because of Bug #1279073")
    def test_show_details_keyboard(self):
        if self.platform_model != "Desktop":
            return
        self.assertThat(lambda: self.main_view.select_many(
            'QQuickFlickable', state="back"), Eventually(Not(Is(True))))
        back_flipped = self.main_view.select_many('QQuickFlipable',
                                                  state="back")
        self.assertEqual(0, len(back_flipped))

        # press space
        self.keyboard.press_and_release('Space')

        # check the displayed info is flipped and only one is flipped
        self.assertThat(lambda: self.main_view.select_many(
            'QQuickFlickable', state="back"), Eventually(Not(Is(True))))
        flipables = self.main_view.select_many('QQuickFlipable')
        self.assertThat(flipables[1].state, Eventually(Equals("back")))
        back_flipped = self.main_view.select_many('QQuickFlipable',
                                                  state="back")
        self.assertEqual(1, len(back_flipped))

        # press space again, hide details
        self.keyboard.press_and_release('Space')

        # check nothing is flipped
        self.assertThat(lambda: self.main_view.select_many(
            'QQuickFlickable', state="back"), Eventually(Not(Is(True))))
        back_flipped = self.main_view.select_many('QQuickFlipable',
                                                  state="back")
        self.assertEqual(0, len(back_flipped))
